/****************************************************************************************
   Temperatur Logger - Software for logging temperature data with the DS18B20 and ESP32
   Written by Sebastian Hinz, https://www.diy-temperature-logger.com
   Licensed under GPLv3. See LICENSE for details.
****************************************************************************************/  
#ifndef _DISPLAY1306_H_
#define _DISPLAY1306_H_

#include "OLEDDisplayUi.h"

const uint8_t emptySymbol[] PROGMEM = {
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000,
    B00000000
};

class sensorOledData;

//////////////////////////////////////////////////////////////////////
enum display1306_state
{
  DISPLAY_UNDEF,
  DISPLAY_TEXT,     // one text page
  DISPLAY_RUN,      // up to nine sensor data pages
};

//////////////////////////////////////////////////////////////////////
// different prio levels of the messages, used to determine how long to display the text and not to overwrite a fatal message with a debug message
typedef enum 
{
  PRIO_UNDEF,
  PRIO_DEBUG,   
  PRIO_INFO,
  PRIO_ERROR,
  PRIO_FATAL,
} messagePriority_t;

//////////////////////////////////////////////////////////////////////
// maps a font to a number of displayed lines
typedef struct 
{
  const uint8_t * fontData;
  int          linesPerDisplay;
} displayLines_t;

//////////////////////////////////////////////////////////////////////
class display1306
{
public:
  display1306();
  static display1306& GetInstance();
  static void RunInstance(void * pvParameters);             // main task function
  
  void addMessage(String newText, messagePriority_t mPrio); // add a message
  void initDone();                                          // common initialising is done, now there are sensor values

  static String m_SensorFound;                              // top line, number of sensors found
 
private:
  void setup();                                             // internal setup function, is called by RunInstance
  void loop();                                              // internal loop function, is called by RunInstance
  
  // different static functions, these functions are called from the display library to display the different pages (one text page, up to nine sensor pages)
  static void drawTopLine(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y);
  static void drawFrameText(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y);
  static void drawFrameSensor0(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 0); }
  static void drawFrameSensor1(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 1); }
  static void drawFrameSensor2(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 2); }
  static void drawFrameSensor3(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 3); }
  static void drawFrameSensor4(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 4); }
  static void drawFrameSensor5(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 5); }
  static void drawFrameSensor6(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 6); }
  static void drawFrameSensor7(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 7); }
  static void drawFrameSensor8(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 8); }
  static void drawFrameSensor9(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) { drawFrameSensor(display, state, x, y, 9); }

  static void drawFrameSensor(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y, int16_t nDisplay);
  
private:
  FrameCallback framesText[1];            // array with function pointers for display libary, one text page
  FrameCallback framesSensor[10];         // array with function pointers for display libary, 10 sensor pages

  display1306_state     m_NewState;       // set this state
  display1306_state     m_State;          // actual state
  bool                  m_bInitDone;      // common initialize is done --> there are sensor values

  static String         m_TextString;       // variables for info/error messages
  unsigned long         m_TextEndTime;      // show message till (millis >= m_TextEndTime)
  messagePriority_t     m_MessagePrio;      // actual message prio
 
  static sensorOledData *m_oledData;        // sensor textes and values

  static const uint8_t * m_FontName;        // font name
  static int             m_FontLines;       // lines per display for this font

  static int            m_actPage;          // actual display page (0..9)
  static int            m_maxPage;          // max page 1, n
  SemaphoreHandle_t     m_xMessageSemaphore;  // semaphore to protect m_TextString, m_TextEndTime, m_MessagePrio
};

#endif
