/****************************************************************************************
   Temperatur Logger - Software for logging temperature data with the DS18B20 and ESP32
   Written by Sebastian Hinz, https://www.diy-temperature-logger.com
   Licensed under GPLv3. See LICENSE for details.
****************************************************************************************/
#ifndef _SENSOR18B20_H_
#define _SENSOR18B20_H_

#include <map>

//////////////////////////////////////////////////////////////////////
// Model IDs
#define DS18S20MODEL 0x10  // also DS1820
#define DS18B20MODEL 0x28
#define DS1822MODEL  0x22
#define DS1825MODEL  0x3B

#define MAX_NUMBER_OF_SENSORS 20   // maximum number of Dallas sensors

//////////////////////////////////////////////////////////////////////
class SensorData
{
public:
  SensorData();
  void Configure();
  void debugPrint();

public:
  byte    m_Addr[8];
  String  m_Name;
  String  m_crc16Name;
  float   m_tempCelcius;
  float   m_CountGesamt;
  float   m_CountCRC1;
  float   m_CountCRC2;
  float   m_CountSensorError;
};

//////////////////////////////////////////////////////////////////////
struct sensorOledData
{
  String  m_LineString[MAX_NUMBER_OF_SENSORS];
  int     m_nLines;
};

//////////////////////////////////////////////////////////////////////
class sensor18b20
{
public:
  sensor18b20();
  static sensor18b20& GetInstance();
  static void RunInstance(void * pvParameters);
  void update();
  sensorOledData* getDisplayText() { return &m_OledData; }
  String getSensorEntryText(int index) { m_LastWrittenText[index] = m_SensorEntryText[index]; return m_SensorEntryText[index]; }
  bool isSensorEntryTextChanged(int index) const { return (m_SensorEntryText[index] != m_LastWrittenText[index]); }
  String getSensorCRC16Id(int index) const { return m_Sensor[index].m_crc16Name; }
  String getWebAppConfigText() const { return m_WebAppConfigText; }
  byte getNumberSensorCnt() const { return m_numberOfFoundSensors; }
  bool isSetupFinished() const { return m_bSetupFinished; } 
  
private:
  void setup();
  void loop();

private:
  bool                        m_bSetupFinished;
  byte                        m_numberOfFoundSensors;
  sensorOledData              m_OledData;
  String                      m_SensorEntryText[MAX_NUMBER_OF_SENSORS];
  String                      m_LastWrittenText[MAX_NUMBER_OF_SENSORS];
  String                      m_WebAppConfigText;
  String                      m_ValidStartTime;
  SensorData                  m_Sensor[MAX_NUMBER_OF_SENSORS];
  std::map<int, SensorData*>  m_SensorOrderOled;
  std::map<int, SensorData*>  m_SensorOrderWeb;
  byte                        m_MeasureIndex;
  byte                        m_ProcessIndex;
};

#endif
