/****************************************************************************************
   Temperatur Logger - Software for logging temperature data with the DS18B20 and ESP32
   Written by Sebastian Hinz, https://www.diy-temperature-logger.com
   Licensed under GPLv3. See LICENSE for details.
****************************************************************************************/
#include <Wire.h>  // Only needed for Arduino 1.6.5 and earlier
#include "SSD1306.h" // alias for `#include "SSD1306Wire.h"`

#include "OLEDDisplayUi.h"

//OLED pins to ESP32 GPIOs via this connecthin:
//OLED_SDA -- GPIO4
//OLED_SCL -- GPIO15
//OLED_RST -- GPIO16

SSD1306  display(0x3c, 5, 4);

OLEDDisplayUi ui     ( &display );

char *pSD = NULL;
char *pSensor1 = NULL;
char *pSensor2 = NULL;

//////////////////////////////////////////////////////////////////////
void drawFrame2(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {
  // Demonstrates the 3 included default sizes. The fonts come from SSD1306Fonts.h file
  // Besides the default fonts there will be a program to convert TrueType fonts into this format
  display->setTextAlignment(TEXT_ALIGN_LEFT);
  display->setFont(ArialMT_Plain_10);
  
  if(pSD == NULL)
  {
    pSD = "Check SD ...";
  }
  if(pSensor1 == NULL)
  {
    pSensor1 = "Check Sensor ...";
  }
  if(pSensor2 == NULL)
  {
    pSensor2 = " ";
  }

  display->drawString(0 + x, 10 + y, pSD);
  display->drawString(0 + x, 24 + y, pSensor1);
  display->drawString(0 + x, 34 + y, pSensor2);
}

// This array keeps function pointers to all frames
// frames are the single views that slide in
FrameCallback frames[] = { drawFrame2 };

// how many frames are there?
int frameCount = 1;

//////////////////////////////////////////////////////////////////////
void setup_1306() {
  pinMode(16,OUTPUT);
  digitalWrite(16, LOW);    // set GPIO16 low to reset OLED
  delay(50); 
  digitalWrite(16, HIGH); // while OLED is running, must set GPIO16 in high
  

	// The ESP is capable of rendering 60fps in 80Mhz mode
	// but that won't give you much time for anything else
	// run it in 160Mhz mode or just set it to 30 fps
  ui.setTargetFPS(60);

	// Customize the active and inactive symbol
 // ui.setActiveSymbol(activeSymbol);
 // ui.setInactiveSymbol(inactiveSymbol);

  // You can change this to
  // TOP, LEFT, BOTTOM, RIGHT
  ui.setIndicatorPosition(BOTTOM);

  // Defines where the first frame is located in the bar.
  ui.setIndicatorDirection(LEFT_RIGHT);

  // You can change the transition that is used
  // SLIDE_LEFT, SLIDE_RIGHT, SLIDE_UP, SLIDE_DOWN
  ui.setFrameAnimation(SLIDE_LEFT);

  // Add frames
  ui.setFrames(frames, frameCount);

  // Add overlays
  //ui.setOverlays(overlays, overlaysCount);

  // Initialising the UI will init the display too.
  ui.init();

  display.flipScreenVertically();

}

//////////////////////////////////////////////////////////////////////
void loop_1306() {
  int remainingTimeBudget = ui.update();

  if (remainingTimeBudget > 0) {
    // You can do some work here
    // Don't do stuff if you are below your
    // time budget.
    delay(remainingTimeBudget);
  }
}

//////////////////////////////////////////////////////////////////////

